#ifndef CLUSTER_H
#define CLUSTER_H

#include "ARR.H"
#include "okcdata.H"

#define IS_CONTOUR0  1<<0
#define IS_CONTOUR1  1<<1
#define IS_MAGNIFIED 1<<2
#define IS_BRUSHED   1<<3
#define IS_BRUSHED_ANY   1<<4

/*Once the number of children of a cluster lower than 
TREEMAP_LIMIT*number of data points of the data set, 
do not draw the details of this cluster on tree map.*/
#define TREEMAP_LIMIT 0.01
struct Cluster 
{
    int parent;
    int num_children;
    ARR<int> child;

    ARR<double> mean;
//jing
    ARR<double> max;
    ARR<double> min;
//jing
    ARR<double> all_mean;

    okcdata::mark_type mark;

    // cfentry of a cluster
    int entries;
    ARR<double> sx;
    double sxx;
    double sqrt_radius;
    double sqrt_diameter;

    double color;
    
    // The dimensions for the rect in tree-map
    double xmin;
    double xmax;
    double ymin;
    double ymax;

    int flags;              
};

struct ClusterTree
{
    char *clusterfile;
    
    ARR<Cluster> nodes;
    int dimension;
    int treemap_limit_number;
    
    ARR<double> min_extents;
    ARR<double> max_extents;

    ClusterTree() :
        clusterfile(0),
        dimension(0)
    {}

    ~ClusterTree() {
        if (clusterfile) delete [] clusterfile;
    }   

    bool ReadClusterTreeFile(const char* fname);
    bool ReadBinaryClusterTreeFile(const char* fname);
	bool ReadClusterTreeFileWithDatabase(const char* fname);
    bool ReadClusterTreeFile2();
    bool ReadClusterTreeFile3();
};

extern ClusterTree *btree;
extern ARR<int> histo;
extern int dep_dim;

extern void hierDeleteClusterTree();
extern void hierInitExtents();
extern void hierInitClusterTree();
extern void hierClearNodeFlags(int bits=0xffffff);
extern void hierMarkContourNodes(double radius, int FLAG);
extern void hierModifyExtentsByBrushParameters();
extern void hierModifyBrushParametersByExtents();
extern void hierMarkBrushedClustersByColor(Cluster* node, double L, double R);
extern void hierMarkBrushedClustersByExtents(Cluster* node);
extern void hierSetMeanOnOff();
extern void hierReorderDimension();


extern void SetupBrushParameters();

#endif
